from contextlib import asynccontextmanager
from typing import Annotated, Any, AsyncIterator

from mcp.server import FastMCP
from mcp.server.fastmcp import Context
from mcp.types import TextContent
from pydantic import Field

from mcp_tracker.mcp.context import AppContext
from mcp_tracker.mcp.errors import TrackerError
from mcp_tracker.mcp.helpers import dump_list, prepare_text_content
from mcp_tracker.mcp.params import IssueID, IssueIDs, QueueID, UserID
from mcp_tracker.settings import Settings
from mcp_tracker.tracker.caching.client import make_cached_protocols
from mcp_tracker.tracker.custom.client import TrackerClient
from mcp_tracker.tracker.proto.fields import GlobalDataProtocol
from mcp_tracker.tracker.proto.issues import IssueProtocol
from mcp_tracker.tracker.proto.queues import QueuesProtocol
from mcp_tracker.tracker.proto.types.queues import Queue
from mcp_tracker.tracker.proto.users import UsersProtocol

settings = Settings()


@asynccontextmanager
async def tracker_lifespan(server: FastMCP) -> AsyncIterator[AppContext]:
    tracker = TrackerClient(
        token=settings.tracker_token,
        cloud_org_id=settings.tracker_cloud_org_id,
        org_id=settings.tracker_org_id,
    )

    queues: QueuesProtocol = tracker
    issues: IssueProtocol = tracker
    fields: GlobalDataProtocol = tracker
    users: UsersProtocol = tracker
    if settings.cache_enabled:
        queues_wrap, issues_wrap, fields_wrap, users_wrap = make_cached_protocols(
            settings.cache_kwargs()
        )
        queues = queues_wrap(queues)
        issues = issues_wrap(issues)
        fields = fields_wrap(fields)
        users = users_wrap(users)

    try:
        yield AppContext(
            queues=queues,
            issues=issues,
            fields=fields,
            users=users,
        )
    finally:
        await tracker.close()


mcp = FastMCP(
    name="Yandex Tracker MCP Server",
    host=settings.host,
    port=settings.port,
    lifespan=tracker_lifespan,
)


def check_issue_id(issue_id: str) -> None:
    queue, _ = issue_id.split("-")
    if settings.tracker_limit_queues and queue not in settings.tracker_limit_queues:
        raise TrackerError(f"Issue `{issue_id}` not found.")


@mcp.tool(
    description="Find all Yandex Tracker queues available to the user (queue is a project in some sense)"
)
async def queues_get_all(
    ctx: Context[Any, AppContext],
) -> TextContent:
    result: list[Queue] = []
    per_page = 100
    page = 1

    while True:
        queues = await ctx.request_context.lifespan_context.queues.queues_list(
            per_page=per_page, page=page
        )
        if len(queues) == 0:
            break

        if settings.tracker_limit_queues:
            queues = [
                queue
                for queue in queues
                if queue.key in set(settings.tracker_limit_queues)
            ]

        result.extend(queues)
        page += 1

    return prepare_text_content(result)


@mcp.tool(
    description="Get local fields for a specific Yandex Tracker queue (queue-specific custom fields)"
)
async def queue_get_local_fields(
    ctx: Context[Any, AppContext],
    queue_id: QueueID,
) -> TextContent:
    if settings.tracker_limit_queues and queue_id not in settings.tracker_limit_queues:
        raise TrackerError(f"Queue `{queue_id}` not found or not allowed.")

    fields = await ctx.request_context.lifespan_context.queues.queues_get_local_fields(
        queue_id
    )
    return prepare_text_content(fields)


@mcp.tool(description="Get all tags for a specific Yandex Tracker queue")
async def queue_get_tags(
    ctx: Context[Any, AppContext],
    queue_id: QueueID,
) -> TextContent:
    if settings.tracker_limit_queues and queue_id not in settings.tracker_limit_queues:
        raise TrackerError(f"Queue `{queue_id}` not found or not allowed.")

    tags = await ctx.request_context.lifespan_context.queues.queues_get_tags(queue_id)
    return prepare_text_content(tags)


@mcp.tool(
    description="Get all global fields available in Yandex Tracker that can be used in issues"
)
async def get_global_fields(
    ctx: Context[Any, AppContext],
) -> TextContent:
    fields = await ctx.request_context.lifespan_context.fields.get_global_fields()
    return prepare_text_content(fields)


@mcp.tool(
    description="Get all statuses available in Yandex Tracker that can be used in issues"
)
async def get_statuses(
    ctx: Context[Any, AppContext],
) -> TextContent:
    statuses = await ctx.request_context.lifespan_context.fields.get_statuses()
    return prepare_text_content(statuses)


@mcp.tool(
    description="Get all issue types available in Yandex Tracker that can be used when creating or updating issues"
)
async def get_issue_types(
    ctx: Context[Any, AppContext],
) -> TextContent:
    issue_types = await ctx.request_context.lifespan_context.fields.get_issue_types()
    return prepare_text_content(issue_types)


@mcp.tool(description="Get a Yandex Tracker issue url by its id")
async def issue_get_url(
    issue_id: IssueID,
) -> TextContent:
    return prepare_text_content({"url": f"https://tracker.yandex.ru/{issue_id}"})


@mcp.tool(description="Get a Yandex Tracker issue by its id")
async def issue_get(
    ctx: Context[Any, AppContext],
    issue_id: IssueID,
) -> TextContent:
    check_issue_id(issue_id)

    issue = await ctx.request_context.lifespan_context.issues.issue_get(issue_id)
    if issue is None:
        raise TrackerError(f"Issue `{issue_id}` not found.")

    return prepare_text_content(issue)


@mcp.tool(description="Get comments of a Yandex Tracker issue by its id")
async def issue_get_comments(
    ctx: Context[Any, AppContext],
    issue_id: IssueID,
) -> TextContent:
    check_issue_id(issue_id)

    comments = await ctx.request_context.lifespan_context.issues.issue_get_comments(
        issue_id
    )
    if comments is None:
        raise TrackerError(f"Issue `{issue_id}` not found.")

    return prepare_text_content(comments)


@mcp.tool(
    description="Get a Yandex Tracker issue related links to other issues by its id"
)
async def issue_get_links(
    ctx: Context[Any, AppContext],
    issue_id: IssueID,
) -> TextContent:
    check_issue_id(issue_id)

    links = await ctx.request_context.lifespan_context.issues.issues_get_links(issue_id)
    if links is None:
        raise TrackerError(f"Issue `{issue_id}` not found.")

    return prepare_text_content(links)


@mcp.tool(description="Find Yandex Tracker issues by queue and/or created date")
async def issues_find(
    ctx: Context[Any, AppContext],
    query: Annotated[
        str,
        Field(
            description=(
                """Search query to filter issues using Yandex Tracker Query Language, Instructions:\n"""
                """# General instructions\n"""
                """1. To search by a specific field use the following syntax: `Description: "some issue description"`\n"""
                """2. Multiple fields should be separated by space: `Description: "some issue description" Created: today()`\n"""
                """3. If you need to specify multiple values for the same field - provide them using comma (,), e.g.: `author: "vpupkin","iivanov"`\n"""
                """4. You may specify multiple conditions and combine them using `AND` and `OR` statements, e.g. `<param_1>: "<value_1>" AND <param_2>: "<value_2>"`\n"""
                """5. You may use brackets for complex logical expressions\n"""
                """6. To find issues with exact string matching in the field use this syntax: `Summary: #"Version 2.0"`. If you need to pass special characters - you must escape them using `\\` symbol\n"""
                """7. To find issues that don't contain the specified text use this syntax: `Summary: !"Version 2.0"`. If you need to pass special characters - you must escape them using `\\` symbol\n"""
                """8. If you need to search by local queue field use the following syntax: `<QUEUE>.<LOCAL_FIELD_KEY>: "<value>", where <QUEUE> is a queue key, <LOCAL_FIELD_KEY> is a local field's key from the `queue_get_local_fields` tool result.\n"""
                """9. For dates use the format YYYY-MM-DD.\n"""
                """10. For numerical values you may use comparison operators (>, <, >=, <=): `<param>: ><value>`.\n"""
                """11. To sort the result specify the `Sort By` directive (you may provide ASC or DESC for the sort order): `"Sort By": Created ASC`.\n"""
                """# Functions\n"""
                """These functions may be used, for example: `Created: week()` - return issues created on the current week"\n"""
                """* `empty()` - empty value\n"""
                """* `notEmpty()` - not empty value\n"""
                """* `now()` - current time\n"""
                """* `today()` - current date\n"""
                """* `week()` - current week\n"""
                """* `month()` - current month\n"""
                """* `quarter()` - current quarter\n"""
                """* `year()` - current year\n"""
                """* `unresolved()` - there is no resolution\n"""
                """# Examples\n"""
                """Find issues in a specific queue: `"Queue": "PROJ"`\n"""
                """Find issues by an assignee: `"Assignee": "Иван Иванов"`\n"""
                """Find issues in specific status: `"Status": "Открыт", "В работе"`\n"""
                """Find issues created in a specific range: `"Created": "2017-01-01".."2017-01-30"`\n"""
                """Find issues created no earlier than 1 week and 1 day before today: `Created: > today() - "1w 1d"`\n"""
            )
        ),
    ],
    page: Annotated[
        int,
        Field(
            description="Page number to return, default is 1",
        ),
    ] = 1,
) -> TextContent:
    per_page = 500

    issues = await ctx.request_context.lifespan_context.issues.issues_find(
        query=query,
        per_page=per_page,
        page=page,
    )

    return prepare_text_content(issues)


@mcp.tool(description="Get worklogs of a Yandex Tracker issue by its id")
async def issue_get_worklogs(
    ctx: Context[Any, AppContext],
    issue_ids: IssueIDs,
) -> TextContent:
    for issue_id in issue_ids:
        check_issue_id(issue_id)

    result: dict[str, Any] = {}
    for issue_id in issue_ids:
        worklogs = await ctx.request_context.lifespan_context.issues.issue_get_worklogs(
            issue_id
        )
        if not worklogs:
            result[issue_id] = []
        else:
            result[issue_id] = dump_list(worklogs)

    return prepare_text_content(result)


@mcp.tool(description="Get attachments of a Yandex Tracker issue by its id")
async def issue_get_attachments(
    ctx: Context[Any, AppContext],
    issue_id: IssueID,
) -> TextContent:
    check_issue_id(issue_id)

    attachments = (
        await ctx.request_context.lifespan_context.issues.issue_get_attachments(
            issue_id
        )
    )
    if attachments is None:
        raise TrackerError(f"Issue `{issue_id}` not found.")

    return prepare_text_content(attachments)


@mcp.tool(
    description="Get information about user accounts registered in the organization"
)
async def users_get_all(
    ctx: Context[Any, AppContext],
    per_page: Annotated[
        int,
        Field(
            description="Number of users per page (default: 50)",
            ge=1,
        ),
    ] = 50,
    page: Annotated[
        int,
        Field(
            description="Page number to return (default: 1)",
            ge=1,
        ),
    ] = 1,
) -> TextContent:
    users = await ctx.request_context.lifespan_context.users.users_list(
        per_page=per_page, page=page
    )
    return prepare_text_content(users)


@mcp.tool(description="Get information about a specific user by login or UID")
async def user_get(
    ctx: Context[Any, AppContext],
    user_id: UserID,
) -> TextContent:
    user = await ctx.request_context.lifespan_context.users.user_get(user_id)
    if user is None:
        raise TrackerError(f"User `{user_id}` not found.")

    return prepare_text_content(user)



from typing import Any

from aiohttp import ClientSession, ClientTimeout
from pydantic import RootModel

from mcp_tracker.tracker.proto.fields import GlobalDataProtocol
from mcp_tracker.tracker.proto.issues import IssueProtocol
from mcp_tracker.tracker.proto.queues import QueuesProtocol
from mcp_tracker.tracker.proto.types.fields import GlobalField, LocalField
from mcp_tracker.tracker.proto.types.issue_types import IssueType
from mcp_tracker.tracker.proto.types.issues import (
    Issue,
    IssueAttachment,
    IssueComment,
    IssueLink,
    Worklog,
)
from mcp_tracker.tracker.proto.types.queues import Queue
from mcp_tracker.tracker.proto.types.statuses import Status
from mcp_tracker.tracker.proto.types.users import User
from mcp_tracker.tracker.proto.users import UsersProtocol

QueueList = RootModel[list[Queue]]
LocalFieldList = RootModel[list[LocalField]]
QueueTagList = RootModel[list[str]]
IssueLinkList = RootModel[list[IssueLink]]
IssueList = RootModel[list[Issue]]
IssueCommentList = RootModel[list[IssueComment]]
WorklogList = RootModel[list[Worklog]]
IssueAttachmentList = RootModel[list[IssueAttachment]]
GlobalFieldList = RootModel[list[GlobalField]]
StatusList = RootModel[list[Status]]
IssueTypeList = RootModel[list[IssueType]]
UserList = RootModel[list[User]]


class TrackerClient(QueuesProtocol, IssueProtocol, GlobalDataProtocol, UsersProtocol):
    def __init__(
        self,
        *,
        token: str,
        org_id: str | None = None,
        base_url: str = "https://api.tracker.yandex.net",
        timeout: float = 10,
        cloud_org_id: str | None = None,
    ):
        headers = {
            "Authorization": f"OAuth {token}",
        }

        if org_id is not None:
            headers["X-Org-ID"] = org_id
        elif cloud_org_id is not None:
            headers["X-Cloud-Org-ID"] = cloud_org_id
        else:
            raise ValueError("Either org_id or cloud_org_id must be provided.")

        self._session = ClientSession(
            base_url=base_url,
            timeout=ClientTimeout(total=timeout),
            headers=headers,
        )

    async def close(self):
        await self._session.close()

    async def queues_list(self, per_page: int = 100, page: int = 1) -> list[Queue]:
        params = {
            "perPage": per_page,
            "page": page,
        }
        async with self._session.get("v3/queues", params=params) as response:
            response.raise_for_status()
            return QueueList.model_validate_json(await response.read()).root

    async def queues_get_local_fields(self, queue_id: str) -> list[LocalField]:
        async with self._session.get(f"v3/queues/{queue_id}/localFields") as response:
            response.raise_for_status()
            return LocalFieldList.model_validate_json(await response.read()).root

    async def queues_get_tags(self, queue_id: str) -> list[str]:
        async with self._session.get(f"v3/queues/{queue_id}/tags") as response:
            response.raise_for_status()
            return QueueTagList.model_validate_json(await response.read()).root

    async def get_global_fields(self) -> list[GlobalField]:
        async with self._session.get("v3/fields") as response:
            response.raise_for_status()
            return GlobalFieldList.model_validate_json(await response.read()).root

    async def get_statuses(self) -> list[Status]:
        async with self._session.get("v3/statuses") as response:
            response.raise_for_status()
            return StatusList.model_validate_json(await response.read()).root

    async def get_issue_types(self) -> list[IssueType]:
        async with self._session.get("v3/issuetypes") as response:
            response.raise_for_status()
            return IssueTypeList.model_validate_json(await response.read()).root

    async def issue_get(self, issue_id: str) -> Issue | None:
        async with self._session.get(f"v3/issues/{issue_id}") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return Issue.model_validate_json(await response.read())

    async def issues_get_links(self, issue_id: str) -> list[IssueLink] | None:
        async with self._session.get(f"v3/issues/{issue_id}/links") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return IssueLinkList.model_validate_json(await response.read()).root

    async def issue_get_comments(self, issue_id: str) -> list[IssueComment] | None:
        async with self._session.get(f"v3/issues/{issue_id}/comments") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return IssueCommentList.model_validate_json(await response.read()).root

    async def issues_find(
        self,
        query: str,
        *,
        per_page: int = 15,
        page: int = 1,
    ) -> list[Issue]:
        params = {
            "perPage": per_page,
            "page": page,
        }

        body: dict[str, Any] = {
            "query": query,
        }

        async with self._session.post(
            "v3/issues/_search", json=body, params=params
        ) as response:
            response.raise_for_status()
            return IssueList.model_validate_json(await response.read()).root

    async def issue_get_worklogs(self, issue_id: str) -> list[Worklog] | None:
        async with self._session.get(f"v3/issues/{issue_id}/worklog") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return WorklogList.model_validate_json(await response.read()).root

    async def issue_get_attachments(
        self, issue_id: str
    ) -> list[IssueAttachment] | None:
        async with self._session.get(f"v2/issues/{issue_id}/attachments") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return IssueAttachmentList.model_validate_json(await response.read()).root

    async def users_list(self, per_page: int = 50, page: int = 1) -> list[User]:
        params: dict[str, str | int] = {
            "perPage": per_page,
            "page": page,
        }
        async with self._session.get("v3/users", params=params) as response:
            response.raise_for_status()
            return UserList.model_validate_json(await response.read()).root

    async def user_get(self, user_id: str) -> User | None:
        async with self._session.get(f"v3/users/{user_id}") as response:
            if response.status == 404:
                return None
            response.raise_for_status()
            return User.model_validate_json(await response.read())



from typing import Protocol

from .types.users import User


class UsersProtocol(Protocol):
    async def users_list(self, per_page: int = 50, page: int = 1) -> list[User]: ...

    async def user_get(self, user_id: str) -> User | None: ...


class UsersProtocolWrap(UsersProtocol):
    def __init__(self, original: UsersProtocol):
        self._original = original



from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity
from mcp_tracker.tracker.proto.types.refs import IssueTypeReference, PriorityReference


class Queue(BaseTrackerEntity):
    id: int
    key: str | None = None
    name: str | None = None
    description: str | None = None
    defaultType: IssueTypeReference | None = None
    defaultPriority: PriorityReference | None = None



from typing import Annotated, Any, Literal

from aiocache import Cache
from aiocache.serializers import PickleSerializer
from pydantic import field_validator
from pydantic_settings import BaseSettings, NoDecode, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8")
    host: str = "0.0.0.0"
    port: int = 8001
    transport: Literal["stdio", "sse", "streamable-http"] = "stdio"
    tracker_token: str
    tracker_cloud_org_id: str | None = None
    tracker_org_id: str | None = None
    tracker_limit_queues: Annotated[list[str] | None, NoDecode] = None
    cache_enabled: bool = False
    cache_redis_endpoint: str = "localhost"
    cache_redis_port: int = 6379
    cache_redis_db: int = 0
    cache_redis_ttl: int | None = 3600

    @field_validator("tracker_limit_queues", mode="before")
    @classmethod
    def decode_numbers(cls, v: str | None) -> list[str] | None:
        if v is None:
            return None
        if not isinstance(v, str):
            raise TypeError(f"Expected str or None, got {type(v)}")

        return [x.strip() for x in v.split(",") if x.strip()]

    def cache_kwargs(self) -> dict[str, Any]:
        return {
            "cache": Cache.REDIS,
            "endpoint": self.cache_redis_endpoint,
            "port": self.cache_redis_port,
            "db": self.cache_redis_db,
            "pool_max_size": 10,
            "serializer": PickleSerializer(),
            "noself": True,
            "ttl": self.cache_redis_ttl,
        }



from pydantic import Field

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity


class IssueType(BaseTrackerEntity):
    id: int = Field(description="Issue type ID")
    version: int = Field(description="Issue type version")
    key: str = Field(description="Issue type key")
    name: str = Field(description="Displayed issue type name")
    description: str | None = Field(default=None, description="Issue type description")



from pydantic import ConfigDict

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity


class User(BaseTrackerEntity):
    model_config = ConfigDict(extra="ignore")

    uid: int
    login: str
    first_name: str | None = None
    last_name: str | None = None
    display: str | None = None
    email: str | None = None
    external: bool | None = None
    dismissed: bool | None = None



from typing import Protocol

from .types.issues import Issue, IssueAttachment, IssueComment, IssueLink, Worklog


class IssueProtocol(Protocol):
    async def issue_get(self, issue_id: str) -> Issue | None: ...
    async def issue_get_comments(self, issue_id: str) -> list[IssueComment] | None: ...
    async def issues_get_links(self, issue_id: str) -> list[IssueLink] | None: ...
    async def issues_find(
        self,
        query: str,
        *,
        per_page: int = 15,
        page: int = 1,
    ) -> list[Issue]: ...
    async def issue_get_worklogs(self, issue_id: str) -> list[Worklog] | None: ...
    async def issue_get_attachments(
        self, issue_id: str
    ) -> list[IssueAttachment] | None: ...


class IssueProtocolWrap(IssueProtocol):
    def __init__(self, original: IssueProtocol):
        self._original = original



from pydantic import BaseModel, ConfigDict, Field


class BaseTrackerEntity(BaseModel):
    self_: str = Field(alias="self", exclude=True)

    model_config = ConfigDict(
        extra="allow",
    )



from typing import Protocol

from .types.fields import GlobalField
from .types.issue_types import IssueType
from .types.statuses import Status


class GlobalDataProtocol(Protocol):
    async def get_global_fields(self) -> list[GlobalField]: ...
    async def get_statuses(self) -> list[Status]: ...
    async def get_issue_types(self) -> list[IssueType]: ...


class GlobalDataProtocolWrap(GlobalDataProtocol):
    def __init__(self, original: GlobalDataProtocol):
        self._original = original



from typing import Any

from pydantic import BaseModel, Field

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity
from mcp_tracker.tracker.proto.types.refs import BaseReference


class FieldSchema(BaseModel):
    """Field schema/data type information"""

    type: str | None = None
    items: str | None = None
    required: bool | None = None


class SuggestProvider(BaseModel):
    type: str | None = None


class QueryProvider(BaseModel):
    type: str | None = None


class OptionsProvider(BaseModel):
    type: str | None = None
    values: Any | None = None


class CategoryRef(BaseReference):
    display: str | None = None


class GlobalField(BaseTrackerEntity):
    """Global field available in Yandex Tracker"""

    id: str
    name: str | None = None
    key: str | None = None
    version: int | None = None
    schema_: FieldSchema | None = Field(
        default=None, alias="schema", serialization_alias="schema"
    )
    readonly: bool | None = None
    options: bool | None = None
    suggest: bool | None = None
    type: str | None = None
    order: int | None = None
    suggestProvider: SuggestProvider | None = None
    optionsProvider: OptionsProvider | None = None
    queryProvider: QueryProvider | None = None
    category: CategoryRef | None = None


class LocalField(GlobalField):
    """Queue-specific local field"""

    description: str | None = None



import datetime

from pydantic import AliasChoices, Field

from mcp_tracker.tracker.proto.types.base import BaseTrackerEntity
from mcp_tracker.tracker.proto.types.mixins import CreatedMixin, CreatedUpdatedMixin
from mcp_tracker.tracker.proto.types.refs import (
    BaseReference,
    ComponentReference,
    IssueReference,
    IssueTypeReference,
    PriorityReference,
    QueueReference,
    StatusReference,
    UserReference,
)


class User(BaseTrackerEntity):
    display: str
    email: str
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    login: str
    uid: int


class Issue(CreatedUpdatedMixin, BaseTrackerEntity):
    unique: str | None = None
    key: str | None = None
    version: int | None = None
    summary: str
    description: str | None = None
    type: IssueTypeReference
    priority: PriorityReference | None = None
    assignee: UserReference | None = None
    status: StatusReference | None = None
    previous_status: StatusReference | None = Field(
        None, validation_alias=AliasChoices("previousStatus", "previous_status")
    )
    deadline: datetime.date | None = None
    components: list[ComponentReference] | None = None
    start: datetime.date | None = None
    end: datetime.date | None = None
    story_points: float | None = Field(
        None, validation_alias=AliasChoices("storyPoints", "story_points")
    )
    tags: list[str] = Field(default_factory=list)
    votes: int | None = None
    queue: QueueReference | None = None


class IssueComment(CreatedUpdatedMixin, BaseTrackerEntity):
    id: int
    long_id: str | None = Field(
        None, validation_alias=AliasChoices("longId", "long_id")
    )
    text: str | None = None
    transport: str | None = None
    text_html: str | None = Field(
        None, validation_alias=AliasChoices("textHtml", "text_html")
    )


class LinkTypeReference(BaseReference):
    id: str
    inward: str | None = None
    outward: str | None = None


class IssueLink(CreatedUpdatedMixin, BaseTrackerEntity):
    id: int
    direction: str | None = None
    type: LinkTypeReference | None = None
    object: IssueReference | None = None
    assignee: UserReference | None = None
    status: StatusReference | None = None


class Worklog(CreatedUpdatedMixin, BaseTrackerEntity):
    id: int
    start: datetime.datetime | None = None
    duration: datetime.timedelta | None = None
    issue: IssueReference | None = None
    comment: str | None = None


class IssueAttachment(CreatedMixin, BaseTrackerEntity):
    id: str
    name: str
    content: str | None = None
    size: int | None = None
    mimetype: str | None = Field(
        None, validation_alias=AliasChoices("mimeType", "mimetype")
    )
    metadata: dict[str, str] | None = None
